'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var superagent = require('superagent');

var _require = require('node-goody'),
    Helper = _require.Helper;

var qs = require('qs');

var BluePermServiceSdk = function () {
  function BluePermServiceSdk(opt) {
    _classCallCheck(this, BluePermServiceSdk);

    var dftOpt = {
      redisCli: null,
      redisPrefix: 'bps',
      appKey: '',
      appSecret: '',
      host: '',
      cache: true,
      minExpires: 60
    };
    this.opts = Object.assign(dftOpt, opt);
    if (!this.opts.appKey || !this.opts.appSecret) throw new Error('[bps]appKey and appSecret is required');
    if (this.opts.cache && (!this.opts.redisCli || !this.opts.redisPrefix || !this.opts.minExpires)) throw new Error('[bps]redisCli, redisPrefix and minExpires is required when cache is set to true');
    if (!this.opts.host) throw new Error('[bps]host of blue-perm-service is required');
  }

  _createClass(BluePermServiceSdk, [{
    key: 'getFullUrl',
    value: function getFullUrl(uri) {
      return this.opts.host + uri;
    }
  }, {
    key: 'handleCache',
    value: async function handleCache(key, fallback) {
      var ttl = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

      if (this.opts.cache) {
        var res = await this.opts.redisCli.get(key);
        if (res !== '') return JSON.parse(res);
      }
      if (!ttl) ttl = Helper.getRandomIntInclusive(this.opts.minExpires, 2 * this.opts.minExpires);
      var data = await fallback().catch(function (err) {
        console.error('request service error:', err.response.text);
        throw err;
      });
      if (this.opts.cache) {
        await this.opts.redisCli.set(key, JSON.stringify(data), 'EX', ttl);
      }
      return data;
    }
  }, {
    key: 'fillRequestData',
    value: async function fillRequestData() {
      var _this = this;

      var obj = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      var cp = JSON.parse(JSON.stringify(obj));
      cp.timestamp = await this.getTimestamp();
      cp.appKey = this.opts.appKey;
      cp.sign = Helper.genSign(cp, function (sortedStr) {
        return Helper.sha256(sortedStr + cp.timestamp + Helper.sha256(_this.opts.appSecret));
      });
      return cp;
    }
  }, {
    key: 'getTimestamp',
    value: async function getTimestamp() {
      var _this2 = this;

      // 5h
      var diff = await this.handleCache([this.opts.redisPrefix, 'time_diff'].join(':'), async function () {
        var res = await superagent.get(_this2.getFullUrl('/ext/api/v1/time'));
        var timestamp = res.body.data.timestamp;
        return parseInt(parseInt(timestamp) - new Date().getTime() / 1000);
      }, 60 * 60 * 5);
      return String(parseInt(new Date().getTime() / 1000) + diff);
    }
  }, {
    key: 'getRolePerms',
    value: async function getRolePerms(roleKey) {
      var _this3 = this;

      return this.handleCache([this.opts.redisPrefix, 'role_perms'].join(':'), async function () {
        var res = await superagent.get(_this3.getFullUrl('/ext/api/v1/role/perms')).query((await _this3.fillRequestData({ roleKey: roleKey })));
        return res.body.data.list || [];
      });
    }
  }, {
    key: 'checkRolePermByKey',
    value: async function checkRolePermByKey(roleKey, permKey) {
      var _this4 = this;

      return this.handleCache([this.opts.redisPrefix, 'check_role_perm_key', roleKey, permKey].join(':'), async function () {
        var res = await superagent.post(_this4.getFullUrl('/ext/api/v1/role/perm/checkKey')).send((await _this4.fillRequestData({ roleKey: roleKey, permKey: permKey })));
        return !!res.body.data.hasPerm;
      });
    }
  }, {
    key: 'checkRolePermByResource',
    value: async function checkRolePermByResource(roleKey, method, uri) {
      var _this5 = this;

      return this.handleCache([this.opts.redisPrefix, 'check_role_perm_res', roleKey, method, uri].join(':'), async function () {
        var res = await superagent.post(_this5.getFullUrl('/ext/api/v1/role/perm/checkResource')).send((await _this5.fillRequestData({ roleKey: roleKey, method: method, path: uri })));
        return !!res.body.data.hasPerm;
      });
    }
  }, {
    key: 'getUserInfo',
    value: async function getUserInfo(bpssid) {
      var _this6 = this;

      return this.handleCache([this.opts.redisPrefix, 'user_info', bpssid].join(':'), async function () {
        var res = await superagent.get(_this6.getFullUrl('/ext/api/v1/user/info')).query((await _this6.fillRequestData({ bpssid: bpssid })));
        return res.body.data;
      });
    }
  }, {
    key: 'getUserPerms',
    value: async function getUserPerms(bpssid) {
      var _this7 = this;

      return this.handleCache([this.opts.redisPrefix, 'user_perms', bpssid].join(':'), async function () {
        var res = await superagent.get(_this7.getFullUrl('/ext/api/v1/user/perms')).query((await _this7.fillRequestData({ bpssid: bpssid })));
        return res.body.data.list || [];
      });
    }
  }, {
    key: 'checkUserPermByKey',
    value: async function checkUserPermByKey(bpssid, permKey) {
      var _this8 = this;

      return this.handleCache([this.opts.redisPrefix, 'check_user_perm_key', bpssid, permKey].join(':'), async function () {
        var res = await superagent.post(_this8.getFullUrl('/ext/api/v1/user/perm/checkKey')).send((await _this8.fillRequestData({ bpssid: bpssid, permKey: permKey })));
        return !!res.body.data.hasPerm;
      });
    }
  }, {
    key: 'checkUserPermByResource',
    value: async function checkUserPermByResource(bpssid, method, uri) {
      var _this9 = this;

      return this.handleCache([this.opts.redisPrefix, 'check_user_perm_res', bpssid, method, uri].join(':'), async function () {
        var res = await superagent.post(_this9.getFullUrl('/ext/api/v1/user/perm/checkResource')).send((await _this9.fillRequestData({ bpssid: bpssid, method: method, path: uri })));
        return !!res.body.data.hasPerm;
      });
    }
  }, {
    key: 'getRolePageUrl',
    value: async function getRolePageUrl(userId) {
      return this.getFullUrl('/ext/page/role') + '?' + qs.stringify((await this.fillRequestData({ userId: userId })));
    }
  }, {
    key: 'getPermPageUrl',
    value: async function getPermPageUrl(userId) {
      return this.getFullUrl('/ext/page/perm') + '?' + qs.stringify((await this.fillRequestData({ userId: userId })));
    }
  }, {
    key: 'getUserPageUrl',
    value: async function getUserPageUrl(userId) {
      return this.getFullUrl('/ext/page/user') + '?' + qs.stringify((await this.fillRequestData({ userId: userId })));
    }
  }]);

  return BluePermServiceSdk;
}();

module.exports = BluePermServiceSdk;